/* basename -- strip directory and suffix from filenames
   Copyright (C) 1990-1997, 1999 Free Software Foundation, Inc.
   Copyright (C) 2000 Dosius Software Co.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/* Usage: basename name [suffix]
   NAME is a pathname; SUFFIX is a suffix to strip from it.

   basename /usr/foo/lossage/functions.l
   => functions.l
   basename /usr/foo/lossage/functions.l .l
   => functions
   basename functions.lisp p
   => functions.lis */

#include <stdio.h>
#define VA_START(args, lastarg) va_start(args, lastarg)
#include <stdlib.h>
#include <string.h>
#include <errno.h>

/* The official name of this program (e.g., no `g' prefix).  */
#define PROGRAM_NAME "basename"
#define AUTHORS "FIXME unknown"

/* The name this program was run with. */
char *program_name;

int error_one_per_line;
unsigned int error_message_count;

extern void error (int status, int errnum, const char *format, ...);
extern void error_at_line (int status, int errnum, const char *fname,
			   unsigned int lineno, const char *format, ...);
extern void (*error_print_progname) (void);
void (*error_print_progname) (void);

void strip_trailing_slashes (char *path)
{
  int last;

  last = strlen (path) - 1;
  while (last > 0 && (path[last] == '/' || path[last] == '\\'))
    path[last--] = '\0';
}

void
error (int status, int errnum, const char *message, ...)
{
  va_list args;

  if (error_print_progname)
    (*error_print_progname) ();
  else
    {
      fflush (stdout);
      fprintf (stderr, "basename: ");
    }

  VA_START (args, message);
  vfprintf (stderr, message, args);
  va_end (args);

  ++error_message_count;
  if (errnum)
    {
      fprintf (stderr, ": %s", strerror (errnum));
    }
  putc ('\n', stderr);
  fflush (stderr);
  if (status)
    exit (status);
}

#ifndef FILESYSTEM_PREFIX_LEN
#define FILESYSTEM_PREFIX_LEN(Filename) 0
#endif

#ifndef ISSLASH
#define ISSLASH(C) (((C) == '/') || ((C) == '\\'))
#endif

/* In general, we can't use the builtin `basename' function if available,
   since it has different meanings in different environments.
   In some environments the builtin `basename' modifies its argument.
   If NAME is all slashes, be sure to return `/'.  */

char * base_name (char const *name)
{
  char const *base = name += FILESYSTEM_PREFIX_LEN (name);
  int all_slashes = 1;
  char const *p;

  for (p = name; *p; p++)
    {
      if (ISSLASH (*p))
	base = p + 1;
      else
	all_slashes = 0;
    }

  /* If NAME is all slashes, arrange to return `/'.  */
  if (*base == '\0' && ISSLASH (*name) && all_slashes)
    --base;

  return (char *) base;
}

void usage (int status)
{
  if (status != 0)
    fprintf (stderr, "Try `%s --help' for more information.\n",
	     program_name);
  else
    {
      printf ("\
Usage: %s NAME [SUFFIX]\n\
  or:  %s OPTION\n\
",
	      program_name, program_name);
      printf ("\
Print NAME with any leading directory components removed.\n\
If specified, also remove a trailing SUFFIX.\n\
\n\
  --help      display this help and exit\n\
  --version   output version information and exit\n\
");
      puts ("\nReport bugs to <bug-sh-utils@gnu.org>.");
    }
  exit (status);
}

/* Remove SUFFIX from the end of NAME if it is there, unless NAME
   consists entirely of SUFFIX. */

static void
remove_suffix (char *name, const char *suffix)
{
  char *np;
  const char *sp;

  np = name + strlen (name);
  sp = suffix + strlen (suffix);

  while (np > name && sp > suffix)
    if (*--np != *--sp)
      return;
  if (np > name)
    *np = '\0';
}

int
main (int argc, char **argv)
{
  char *name;

  program_name = argv[0];
if (!strcmp(argv[1],"--help")) usage(0);
if (!strcmp(argv[1],"--version")) {printf ("Dosius/GNU basename 2.0\n");return;}
  if (argc > 1 && !strcmp (argv[1], "--"))
    {
      --argc;
      ++argv;
    }

  if (argc == 1 || argc > 3)
    {
      error (0, 0, (argc == 1 ? "too few arguments"
                    : "too many arguments"));
      usage (1);
    }

  strip_trailing_slashes (argv[1]);

  name = base_name (argv[1]);

  if (argc == 3)
    remove_suffix (name, argv[2]);

  puts (name);

  exit (0);
}
